<?php

GFForms::include_feed_addon_framework();

class GFHelpScout extends GFFeedAddOn {

	protected $_version = GF_HELPSCOUT_VERSION;
	protected $_min_gravityforms_version = '1.9.11';
	protected $_slug = 'gravityformshelpscout';
	protected $_path = 'gravityformshelpscout/helpscout.php';
	protected $_full_path = __FILE__;
	protected $_url = 'http://www.gravityforms.com/';
	protected $_title = 'Gravity Forms Help Scout Add-On';
	protected $_short_title = 'Help Scout';

	// Members plugin integration
	protected $_capabilities = array( 'gravityforms_helpscout', 'gravityforms_helpscout_uninstall' );

	// Permissions
	protected $_capabilities_settings_page = 'gravityforms_helpscout';
	protected $_capabilities_form_settings = 'gravityforms_helpscout';
	protected $_capabilities_uninstall = 'gravityforms_helpscout_uninstall';
	protected $_enable_rg_autoupgrade = true;

	protected $api = null;
	private static $_instance = null;

	public static function get_instance() {

		if ( self::$_instance == null ) {
			self::$_instance = new GFHelpScout();
		}

		return self::$_instance;

	}

	/* Plugin Settings */
	public function plugin_settings_fields() {

		return array(
			array(
				'title'       => '',
				'description' => $this->plugin_settings_description(),
				'fields'      => array(
					array(
						'name'              => 'api_key',
						'label'             => __( 'API Key', 'gravityformshelpscout' ),
						'type'              => 'text',
						'class'             => 'medium',
						'feedback_callback' => array( $this, 'initialize_api' )
					),
					array(
						'type'     => 'save',
						'messages' => array(
							'success' => __( 'Help Scout settings have been updated.', 'gravityformshelpscout' )
						),
					),
				),
			),
		);

	}

	/* Prepare plugin settings description */
	public function plugin_settings_description() {

		$description = '<p>';
		$description .= sprintf(
			__( 'HelpScout makes it easy to provide your customers with a great support experience. Use Gravity Forms to collect customer information and automatically create a new Help Scout conversation. If you don\'t have a Help Scout account, you can %1$s sign up for one here.%2$s', 'gravityformshelpscout' ),
			'<a href="http://www.helpscout.net/" target="_blank">', '</a>'
		);
		$description .= '</p>';

		if ( ! $this->initialize_api() ) {

			$description .= '<p>';
			$description .= __( 'Gravity Forms Help Scout Add-On requires your API Key. You can find your API Key by visiting the API Keys page under Your Profile.', 'gravityformshelpscout' );
			$description .= '</p>';

		}

		return $description;

	}

	/* Setup Feed Settings Fields */
	public function feed_settings_fields() {

		$general_settings = array(
			'title'  => '',
			'fields' => array(
				array(
					'name'          => 'feed_name',
					'type'          => 'text',
					'required'      => true,
					'class'         => 'medium',
					'label'         => __( 'Name', 'gravityformshelpscout' ),
					'tooltip'       => '<h6>' . __( 'Name', 'gravityformshelpscout' ) . '</h6>' . __( 'Enter a feed name to uniquely identify this setup.', 'gravityformshelpscout' ),
					'default_value' => $this->get_default_feed_name()
				),
				array(
					'name'     => 'mailbox',
					'type'     => 'select',
					'required' => true,
					'choices'  => $this->mailboxes_for_feed_setting(),
					'onchange' => "jQuery(this).parents('form').submit();",
					'label'    => __( 'Destination Mailbox', 'gravityformshelpscout' ),
					'tooltip'  => '<h6>' . __( 'Destination Mailbox', 'gravityformshelpscout' ) . '</h6>' . __( 'Select the Help Scout Mailbox this form entry will be sent to.', 'gravityformshelpscout' )
				),
				array(
					'name'       => 'user',
					'type'       => 'select',
					'dependency' => 'mailbox',
					'choices'    => $this->users_for_feed_settings(),
					'label'      => __( 'Assign To User', 'gravityformshelpscout' ),
					'tooltip'    => '<h6>' . __( 'Assign To User', 'gravityformshelpscout' ) . '</h6>' . __( 'Choose the Help Scout User this form entry will be assigned to.', 'gravityformshelpscout' )
				),
			),
		);


		$message_settings = array(
			'title'      => __( 'Message Details', 'gravityformshelpscout' ),
			'dependency' => 'mailbox',
			'fields'     => array(
				array(
					'name'          => 'status',
					'type'          => 'select',
					'choices'       => $this->status_types_for_feed_setup(),
					'label'         => __( 'Message Status', 'gravityformshelpscout' ),
				),
				array(
					'name'          => 'type',
					'type'          => 'select',
					'choices'       => $this->message_types_for_feed_setup(),
					'label'         => __( 'Message Type', 'gravityformshelpscout' ),
				),
				array(
					'name'          => 'customer_email',
					'type'          => 'field_select',
					'required'      => true,
					'label'         => __( 'Customer\'s Email Address', 'gravityformshelpscout' ),
					'default_value' => $this->get_first_field_by_type( 'email' ),
					'args'          => array(
						'input_types' => array( 'email', 'hidden' )
					)
				),
				array(
					'name'          => 'customer_first_name',
					'type'          => 'field_select',
					'label'         => __( 'Customer\'s First Name', 'gravityformshelpscout' ),
					'default_value' => $this->get_first_field_by_type( 'name', 3 ),
				),
				array(
					'name'          => 'customer_last_name',
					'type'          => 'field_select',
					'label'         => __( 'Customer\'s Last Name', 'gravityformshelpscout' ),
					'default_value' => $this->get_first_field_by_type( 'name', 6 ),
				),
				array(
					'name'          => 'tags',
					'type'          => 'text',
					'label'         => __( 'Tags', 'gravityformshelpscout' ),
					'class'         => 'medium merge-tag-support mt-position-right mt-hide_all_fields',
				),
				array(
					'name'          => 'subject',
					'type'          => 'text',
					'required'      => true,
					'class'         => 'medium merge-tag-support mt-position-right mt-hide_all_fields',
					'label'         => __( 'Subject', 'gravityformshelpscout' ),
					'default_value' => 'New submission from {form_title}',
				),
				array(
					'name'          => 'body',
					'type'          => 'textarea',
					'required'      => true,
					'use_editor'    => true,
					'class'         => 'large',
					'label'         => __( 'Message Body', 'gravityformshelpscout' ),
					'default_value' => '{all_fields}'
				),
				array(
					'name'          => 'auto_reply',
					'type'          => 'checkbox',
					'label'         => __( 'Auto Reply', 'gravityformshelpscout' ),
					'choices'       => array(
						array(
							'name'  => 'auto_reply',
							'label' => __( 'Send Help Scout auto reply when message is created', 'gravityformshelpscout' ),
						)
					)
				),
			)
		);

		$file_fields_for_feed = $this->file_fields_for_feed_setup();

		if ( ! empty ( $file_fields_for_feed ) ) {

			$message_settings['fields'][] = array(
				'name'    => 'attachments',
				'type'    => 'checkbox',
				'label'   => __( 'Attachments', 'gravityformshelpscout' ),
				'choices' => $file_fields_for_feed
			);

		}

		if ( apply_filters( 'gform_helpscout_enable_cc', false ) ) {

			$message_settings['fields'][] = array(
				'name'     => 'cc',
				'type'     => 'text',
				'required' => true,
				'class'    => 'medium merge-tag-support mt-position-right mt-hide_all_fields',
				'label'    => __( 'CC', 'gravityformshelpscout' ),
			);

		}

		if ( apply_filters( 'gform_helpscout_enable_bcc', false ) ) {

			$message_settings['fields'][] = array(
				'name'     => 'bcc',
				'type'     => 'text',
				'required' => true,
				'class'    => 'medium merge-tag-support mt-position-right mt-hide_all_fields',
				'label'    => __( 'BCC', 'gravityformshelpscout' ),
			);

		}

		$conditional_settings = array(
			'title'  => __( 'Feed Conditional Logic', 'gravityformshelpscout' ),
			'fields' => array(
				array(
					'name'           => 'feed_ondition',
					'type'           => 'feed_condition',
					'dependency'     => 'mailbox',
					'label'          => __( 'Conditional Logic', 'gravityformshelpscout' ),
					'checkbox_label' => __( 'Enable', 'gravityformshelpscout' ),
					'instructions'   => __( 'Export to Help Scout if', 'gravityformshelpscout' ),
					'tooltip'        => '<h6>' . __( 'Conditional Logic', 'gravityformshelpscout' ) . '</h6>' . __( 'When conditional logic is enabled, form submissions will only be exported to Help Scout when the condition is met. When disabled, all form submissions will be posted.', 'gravityformshelpscout' )
				),
			),
		);

		return array( $general_settings, $message_settings, $conditional_settings );

	}

	/* Get Help Scout Mailboxes for feed setup */
	public function mailboxes_for_feed_setting() {

		/* Setup initial choices array. */
		$choices = array(
			array(
				'label' => __( 'Choose A Mailbox', 'gravityformshelpscout' ),
				'value' => ''
			)
		);

		/* If Help Scout instance is not initialized, return choices array. */
		if ( ! $this->initialize_api() ) {
			return $choices;
		}

		/* Get the Help Scout mailboxes. */
		$mailboxes = $this->api->getMailboxes( 99, array( 'id', 'name' ) );

		/* If there are mailboxes, add them to the choices array. */
		if ( $mailboxes ) {

			foreach ( $mailboxes->items as $mailbox ) {

				$choices[] = array(
					'label' => $mailbox->name,
					'value' => $mailbox->id
				);

			}

		}

		return $choices;

	}

	/* Get Help Scout Users for feed setup */
	public function users_for_feed_settings() {

		/* Setup initial choice. */
		$choices = array(
			array(
				'label' => __( 'Do Not Assign', 'gravityformshelpscout' ),
				'value' => ''
			)
		);

		/* If Help Scout instance is not initialized, return choices array. */
		if ( ! $this->initialize_api() ) {
			return $choices;
		}

		/* Get current mailbox value. */
		$feed    = $this->get_current_feed();
		$mailbox = $feed ? $feed['meta']['mailbox'] : rgpost( '_gaddon_setting_mailbox' );

		if ( ! empty( $_POST ) ) {
			$mailbox = rgpost( '_gaddon_setting_mailbox' );
		}

		/* If no mailbox is set, return choices. */
		if ( rgblank( $mailbox ) ) {
			return $choices;
		}

		/* Get the users for mailbox and add to choices array. */
		try {

			$users = $this->api->getUsersForMailbox( $mailbox, 1, array( 'id', 'firstName', 'lastName' ) );

		} catch ( Exception $e ) {

			$this->log_error( __METHOD__ . '(): Failed to set get users for mailbox; ' . $e->getMessage() );

			return $choices;

		}

		if ( $users ) {

			foreach ( $users->items as $user ) {

				$choices[] = array(
					'label' => $user->firstName . ' ' . $user->lastName,
					'value' => $user->id
				);

			}

		}

		return $choices;

	}

	/* Get Help Scout status types for feed setup */
	public function status_types_for_feed_setup() {

		return array(
			array(
				'label' => __( 'Active', 'gravityformshelpscout' ),
				'value' => 'active',
			),
			array(
				'label' => __( 'Pending', 'gravityformshelpscout' ),
				'value' => 'pending',
			),
			array(
				'label' => __( 'Closed', 'gravityformshelpscout' ),
				'value' => 'closed',
			),
			array(
				'label' => __( 'Spam', 'gravityformshelpscout' ),
				'value' => 'spam',
			)
		);

	}

	/* Get Help Scout message types for feed setup */
	public function message_types_for_feed_setup() {

		return array(
			array(
				'label' => __( 'Email', 'gravityformshelpscout' ),
				'value' => 'email',
			),
			array(
				'label' => __( 'Chat', 'gravityformshelpscout' ),
				'value' => 'chat',
			),
			array(
				'label' => __( 'Phone', 'gravityformshelpscout' ),
				'value' => 'phone',
			),
		);

	}

	/* Get file fields for feed setup */
	public function file_fields_for_feed_setup() {

		/* Setup choices array. */
		$choices = array();

		/* Get the form. */
		$form = GFAPI::get_form( rgget( 'id' ) );

		/* Get file fields for the form. */
		$file_fields = GFCommon::get_fields_by_type( $form, array( 'fileupload' ) );

		if ( ! empty ( $file_fields ) ) {

			foreach ( $file_fields as $field ) {

				$choices[] = array(
					'name'          => 'attachments[' . $field->id . ']',
					'label'         => $field->label,
					'default_value' => 0,
				);

			}

		}

		return $choices;

	}

	/* Set feed creation control */
	public function can_create_feed() {
		
		return $this->initialize_api();
		
	}

	/* Setup feed list columns */
	public function feed_list_columns() {

		return array(
			'feed_name' => __( 'Name', 'gravityformshelpscout' ),
			'mailbox'   => __( 'Mailbox', 'gravityformshelpscout' ),
			'user'      => __( 'User', 'gravityformshelpscout' )
		);

	}

	/* Feed List Column: Mailbox */
	public function get_column_value_mailbox( $feed ) {

		/* If Help Scout instance is not initialized, return mailbox ID. */
		if ( ! $this->initialize_api() ) {
			return $feed['meta']['mailbox'];
		}

		try {

			$mailbox = $this->api->getMailbox( $feed['meta']['mailbox'] );

			return $mailbox->getName();

		} catch ( Exception $e ) {

			return $feed['meta']['mailbox'];

		}

	}

	/* Feed List Column: User */
	public function get_column_value_user( $feed ) {

		/* If no user ID is set, return not assigned. */
		if ( rgblank( $feed['meta']['user'] ) ) {
			return __( 'No User Assigned', 'gravityformshelpscout' );
		}

		/* If Help Scout instance is not initialized, return user ID. */
		if ( ! $this->initialize_api() ) {
			return $feed['meta']['user'];
		}

		try {

			$user = $this->api->getUser( $feed['meta']['user'] );

			return $user->getFirstName() . ' ' . $user->getLastName();

		} catch ( Exception $e ) {

			return $feed['meta']['user'];

		}

	}

	/* Process feed */
	public function process_feed( $feed, $entry, $form ) {

		/* If Help Scout instance is not initialized, exit. */
		if ( ! $this->initialize_api() ) {

			$this->log_error( __METHOD__ . '(): Failed to set up the API.' );

			return;

		}

		/* Prepare needed information. */
		$data = array(
			'email'       => $this->get_field_value( $form, $entry, $feed['meta']['customer_email'] ),
			'first_name'  => $this->get_field_value( $form, $entry, $feed['meta']['customer_first_name'] ),
			'last_name'   => $this->get_field_value( $form, $entry, $feed['meta']['customer_last_name'] ),
			'subject'     => GFCommon::replace_variables( $feed['meta']['subject'], $form, $entry ),
			'body'        => GFCommon::replace_variables( $feed['meta']['body'], $form, $entry ),
			'attachments' => array(),
			'tags'        => GFCommon::replace_variables( $feed['meta']['tags'], $form, $entry )
		);

		/* If the email address is empty, exit. */
		if ( GFCommon::is_invalid_or_empty_email( $data['email'] ) ) {

			$this->log_error( __METHOD__ . "(): Email address must be provided." );

			return false;

		}

		/* Setup the mailbox for this conversation */
		$mailbox = new \HelpScout\model\ref\MailboxRef();
		$mailbox->setId( $feed['meta']['mailbox'] );

		/* Create the customer object */
		$customer = $this->api->getCustomerRefProxy( null, $data['email'] );
		$customer->setFirstName( $data['first_name'] );
		$customer->setLastName( $data['last_name'] );

		/* Create the conversation object */
		$conversation = new \HelpScout\model\Conversation();
		$conversation->setSubject( $data['subject'] );
		$conversation->setMailbox( $mailbox );
		$conversation->setCustomer( $customer );
		$conversation->setCreatedBy( $customer );
		$conversation->setType( $feed['meta']['type'] );

		/* Create the message thread */
		if ( gf_apply_filters( 'gform_helpscout_process_body_shortcodes', $form['id'], false, $form, $feed ) ) {
			$data['body'] = do_shortcode( $data['body'] );
		}
		$thread = new \HelpScout\model\thread\Customer();
		$thread->setCreatedBy( $customer );
		$thread->setBody( $data['body'] );
		$thread->setStatus( $feed['meta']['status'] );

		/* Assign this conversation to user if set */
		if ( ! rgempty( 'user', $feed['meta'] ) ) {

			$user = new \HelpScout\model\ref\PersonRef();
			$user->setId( $feed['meta']['user'] );
			$user->setType( 'user' );
			$thread->setAssignedTo( $user );

		}

		/* If feed has an attachments field assign, process the attachments. */
		if ( ! empty( $feed['meta']['attachments'] ) ) {

			$attachment_fields = array_keys( $feed['meta']['attachments'] );

			$attachment_files = array();

			foreach ( $attachment_fields as $attachment_field ) {

				$field_value = $this->get_field_value( $form, $entry, $attachment_field );
				$field_value = $this->is_json( $field_value ) ? json_decode( $field_value, true ) : $field_value;

				if ( empty( $field_value ) ) {
					continue;
				}

				if ( is_array( $field_value ) ) {

					$attachment_files = array_merge( $attachment_files, $field_value );

				} else {

					$attachment_files[] = $field_value;

				}

			}

			if ( ! empty( $attachment_files ) ) {

				$attachments = $this->process_feed_attachments( $attachment_files );
				$thread->setAttachments( $attachments );

			}

		}

		/* Add tags to conversation */
		$tags = ! empty( $data['tags'] ) ? array_map( 'trim', explode( ',', $data['tags'] ) ) : array();
		$tags = gf_apply_filters( 'gform_helpscout_tags', $form['id'], $tags, $feed, $entry, $form );

		if ( ! empty( $tags ) ) {

			$conversation->setTags( $tags );

		}

		/* Add CC and BCC support if set. */
		if ( isset( $feed['meta']['cc'] ) ) {

			$data['cc'] = GFCommon::replace_variables( $feed['meta']['cc'], $form, $entry );
			$data['cc'] = ( is_array( $data['cc'] ) ) ? $data['cc'] : explode( ',', $data['cc'] );

			if ( ! empty( $data['cc'] ) ) {

				$thread->setCcList( $data['cc'] );

			}

		}

		if ( isset( $feed['meta']['bcc'] ) ) {

			$data['bcc'] = GFCommon::replace_variables( $feed['meta']['bcc'], $form, $entry );
			$data['bcc'] = ( is_array( $data['bcc'] ) ) ? $data['bcc'] : explode( ',', $data['bcc'] );

			if ( ! empty( $data['bcc'] ) ) {

				$thread->setCcList( $data['bcc'] );

			}

		}

		/* Assign the message thread to the conversation */
		$conversation->setThreads( array( $thread ) );

		/* Set thread count to 1 so Help Scout will include the conversation in the mailbox folder count */
		$conversation->setThreadCount( 1 );

		$this->log_debug( __METHOD__ . "(): Conversation to be created => " . print_r( $conversation, true ) );

		try {

			$auto_reply = ( rgars( $feed, 'meta/auto_reply' ) == '1' );

			/* Create the conversation. */
			$this->api->createConversation( $conversation, false, $auto_reply, true );

			gform_update_meta( $entry['id'], 'helpscout_conversation_id', $conversation->getId() );

			/* Log that conversation was created. */
			$this->log_debug( __METHOD__ . "(): Conversation has been created." );

			return true;

		} catch ( Exception $e ) {

			/* Log that conversation was not created. */
			$this->log_error( __METHOD__ . "(): Conversation was not created; {$e->getMessage()}" );

			return false;

		}

	}

	/* Process attachments for feed */
	public function process_feed_attachments( $files ) {

		/* Prepare attachments array. */
		$attachments = array();

		/* If Help Scout instance is not initialized, return attachments. */
		if ( ! $this->initialize_api() ) {
			return $attachments;
		}

		/* If there are no files, return. */
		if ( rgblank( $files ) ) {
			return $attachments;
		}

		/* Prepare attachment and add to array */
		foreach ( $files as $file ) {

			/* Get the file name and location of the file */
			$file_name     = basename( $file );
			$file_location = str_replace( WP_CONTENT_URL, WP_CONTENT_DIR, $file );

			/* Get the file's mime type */
			$file_info      = finfo_open( FILEINFO_MIME_TYPE );
			$file_mime_type = finfo_file( $file_info, $file_location );
			finfo_close( $file_info );

			/* Prepare the attachment object. */
			$attachment = new \HelpScout\model\Attachment();
			$attachment->setFileName( $file_name );
			$attachment->setMimeType( $file_mime_type );
			$attachment->setData( file_get_contents( $file_location ) );

			/* Create the attachment. */
			try {

				$this->api->createAttachment( $attachment );

				$attachments[] = $attachment;

			} catch ( Exception $e ) {

				$this->log_error( __METHOD__ . "(): Unable to upload attachment; {$e->getMessage()}" );

			}

		}

		return $attachments;

	}

	/* Checks validity of Help Scout API credentials and initializes API if valid. */
	public function initialize_api() {

		if ( ! is_null( $this->api ) ) {
			return true;
		}

		/* Load the API library. */
		if ( ! class_exists( 'HelpScout\ApiClient' ) ) {
			require_once 'includes/api/ApiClient.php';
		}

		/* Get the API Key. */
		$api_key = $this->get_plugin_setting( 'api_key' );

		/* If the API Key is empty, do not run a validation check. */
		if ( rgblank( $api_key ) ) {
			return null;
		}

		$this->log_debug( __METHOD__ . "(): Validating login for API Info for {$api_key}." );

		/* Setup a new Help Scout object with the API credentials. */
		$help_scout = HelpScout\ApiClient::getInstance();
		$help_scout->setKey( $api_key );

		try {

			/* Make a test request. */
			$help_scout->getMailboxes();

			/* Assign API object to class. */
			$this->api = $help_scout;

			/* Log that test passed. */
			$this->log_debug( __METHOD__ . '(): API credentials are valid.' );

			return true;

		} catch ( Exception $e ) {

			/* Log that test failed. */
			$this->log_error( __METHOD__ . '(): API credentials are invalid; ' . $e->getMessage() );

			return false;

		}

	}

}
